//
//  Nylox
//  Copyright (C) 2006-2008 Agustin Alvarez
// 
//  This program is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
// 
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
//  for more details.
//  
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  K32Descriptor.c
//


#include "Includes/Descriptor.h"
#include <Libc.h>


// Global Descriptor Tables
static i386_GDTDescriptor GDT[GDT_SIZE];
static i386_IDTDescriptor IDT[IDT_SIZE];


void DEFAULT_HANDLE_EXCEPTIONS();

/** Default Int Handle **/
asm (
   ".globl _DEFAULT_HANDLE_EXCEPTIONS         \n"
   "_DEFAULT_HANDLE_EXCEPTIONS:               \n"
   "   iret                \n"
);

int i386IDTInitialize()
{
  int X;
  for(X=0;X<IDT_SIZE;X++)
     if(i386IDTSet(X,I86_IDT_DESC_PRESENT | I86_IDT_DESC_BIT32,0x8,DEFAULT_HANDLE_EXCEPTIONS) == IDT_ALLOC_INVALID)
	   return IDT_ALLOC_INVALID;	
	   
  return IDT_ALLOC_SUCCESFULL;
}
  
int i386GDTInitialize()
{
  // This function should be called by the kernel's initialization function,
  // before multitasking is enabled.  This will set up the initial Global
  // Descriptor Table (GDT) 
  
  int Status;
   
  //Free The Descriptors
  nyloxMemClear((i386_GDTDescriptor *)GDT,sizeof(i386_GDTDescriptor)*GDT_SIZE);
  nyloxMemClear((i386_IDTDescriptor *)IDT,sizeof(i386_IDTDescriptor)*IDT_SIZE);
  
  // Null Selector
  i386GDTSet(0x0,0x0,0x0,0x0,0x0);
  
  // Make the global, privileged code segment descriptor
  Status = i386GDTSet(0x8,0,0xFFFFFFFF,I86_GDT_DESC_READWRITE|I86_GDT_DESC_EXEC_CODE|I86_GDT_DESC_CODEDATA|I86_GDT_DESC_MEMORY,
					  I86_GDT_GRAND_4K | I86_GDT_GRAND_32BIT | I86_GDT_GRAND_LIMITHI_MASK);
  if(Status == 0)
    return Status;
  // Make the global, privileged data segment descriptor
  Status = i386GDTSet(0x10,0,0xFFFFFFFF,I86_GDT_DESC_READWRITE|I86_GDT_DESC_CODEDATA|I86_GDT_DESC_MEMORY,					   
					  I86_GDT_GRAND_4K | I86_GDT_GRAND_32BIT | I86_GDT_GRAND_LIMITHI_MASK);
  if(Status == 0)
    return Status;

  // Make the global, privileged stack segment descriptor
  Status = i386GDTSet(0x18,0,0xFFFFFFFF,I86_GDT_DESC_READWRITE|I86_GDT_DESC_CODEDATA|I86_GDT_DESC_MEMORY,					   
					  I86_GDT_GRAND_4K | I86_GDT_GRAND_32BIT | I86_GDT_GRAND_LIMITHI_MASK);

  if(Status == 0)
    return Status;
  
  // Now we can install our new GDT
  kernelProcessorSetGDT((void*)GDT,(GDT_SIZE*8));
  // Now we can install our new IDT
  kernelProcessorSetIDT((void*)IDT,(IDT_SIZE*8));
  
  return GDT_ALLOC_SUCCESFULL;
}




int i386GDTSet(volatile i386_Selector Selector, volatile void *Base,unsigned Size,unsigned short Access,unsigned short Gran)
{
  // This function is mostly internal, and is used to change a descriptor
  // in a descriptor table.  All of the relevant information should be 
  // passed in as parameters, and they will then be added to the table
  
  int EntryNumber;
  
  EntryNumber = (Selector >> 3);
  
  //ClearMem
  nyloxMemClear((i386_GDTDescriptor *)&GDT[EntryNumber],sizeof(i386_GDTDescriptor));
  
  //Memoria Base
  GDT[EntryNumber].BaseL = (unsigned short)((unsigned)Base & 0x0000FFFF);
  GDT[EntryNumber].BaseM = (unsigned char)(((unsigned)Base >> 0x10) & 0x000000FF);
  GDT[EntryNumber].BaseH = (unsigned char)(((unsigned)Base >> 24) & 0x000000FF);
  
  //Limite
  GDT[EntryNumber].Limit = (unsigned short)(Size & 0x0000FFFF);

  //Bytes De Atributos
  GDT[EntryNumber].Flags = Access;
  GDT[EntryNumber].Grand = (unsigned char)((Size >> 0x10) & 0x0000000F);
  GDT[EntryNumber].Grand |= Gran & 0x000000F0;
  
  return GDT_ALLOC_SUCCESFULL;
}

int i386IDTSet(volatile i386_Selector INTNumber,unsigned short Flags,unsigned short Sel,void (*Address))
{
  // This function can be used to install an interrupt gate descriptor
  // in the interrupt descriptor table.  Takes the number of the interrupt
  // as the first parameter, and the address of the Interrupt Service
  // Routine (ISR) as the second.  Returns 0 on success, negative otherwise
  
  if(INTNumber >= IDT_SIZE)
    return IDT_ALLOC_INVALID;
  
  if(Address == 0x0)
    return IDT_ALLOC_INVALID;
	
  //ClearMem
  nyloxMemClear((i386_IDTDescriptor *)&IDT[INTNumber],sizeof(i386_IDTDescriptor));
  // Now we can begin constructing the descriptor.  Start working through
  // each of the descriptor's bytes.
  IDT[INTNumber].BaseL = (unsigned long int)Address & 0xFFFF;
  IDT[INTNumber].BaseH = (unsigned long int)Address >> 0x10;
  //Memoria Base
  IDT[INTNumber].Reserved  = 0x0;
  //Bytes De Atributos
  IDT[INTNumber].Flags = (unsigned char) Flags;
  IDT[INTNumber].Sel   = Sel;

  return IDT_ALLOC_SUCCESFULL;
}
